Package: callr
Title: Call R from R
Version: 3.8.0
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-7098-9676")),
    person("Winston", "Chang", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49")),
    person("Ascent Digital Services", role = c("cph", "fnd"))
  )
Description: It is sometimes useful to perform a computation in a separate
    R process, without affecting the current R process at all.  This
    packages does exactly that.
License: MIT + file LICENSE
URL: https://callr.r-lib.org, https://github.com/r-lib/callr
BugReports: https://github.com/r-lib/callr/issues
Depends: R (>= 3.4)
Imports: otel (>= 0.2.0), processx (>= 3.6.1), R6, utils
Suggests: asciicast (>= 2.3.1), carrier, cli (>= 1.1.0), otelsdk (>=
        0.2.0), ps, rprojroot, spelling, testthat (>= 3.2.0), withr (>=
        2.3.0)
Config/Needs/website: r-lib/asciicast, glue, htmlwidgets, igraph,
        tibble, tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-28
Encoding: UTF-8
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-05 08:20:00 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-7098-9676>),
  Winston Chang [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>),
  Ascent Digital Services [cph, fnd]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-05 11:40:13 UTC
Built: R 4.6.0; ; 2026-06-05 15:30:17 UTC; unix
