Package: csomag
Title: Test Package for 'callr'
Version: 0.0.0.9999
Authors@R: c(
    person("Gábor", "Csárdi", role = c("aut", "cre", "cph"),
           email = "csardi.gabor@gmail.com",
	   comment = c(ORCID = "0000-0001-7098-9676")))
Description: Test package for 'callr'.
License: MIT + file LICENSE
LazyData: true
Encoding: UTF-8
Language: en-US
