# errors

    Code
      load_client_lib()
    Condition
      Error in `sofile_in_processx()`:
      ! Cannot find client file

# errors 2

    Code
      load_client_lib(sofile)
    Condition
      Error in `dyn.load()`:
      !  after ooops

