% Generated by roxygen2: do not edit by hand % Please edit documentation in R/rscript.R \name{rscript_process} \alias{rscript_process} \title{External \code{Rscript} process} \description{ An \verb{Rscript script.R} command that runs in the background. This is an R6 class that extends the \link[processx:process]{processx::process} class. } \examples{ \dontshow{if (FALSE) withAutoprint(\{ # examplesIf} options <- rscript_process_options(script = "script.R") rp <- rscript_process$new(options) rp$wait() rp$read_output_lines() \dontshow{\}) # examplesIf} } \section{Super class}{ \code{\link[processx:process]{processx::process}} -> \code{rscript_process} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-rscript_process-initialize}{\code{rscript_process$new()}} \item \href{#method-rscript_process-cleanup}{\code{rscript_process$cleanup()}} \item \href{#method-rscript_process-finalize}{\code{rscript_process$finalize()}} \item \href{#method-rscript_process-clone}{\code{rscript_process$clone()}} } } \if{html}{\out{
Inherited methods

- `processx::process$as_ps_handle()`
- `processx::process$format()`
- `processx::process$get_cmdline()`
- `processx::process$get_cpu_times()`
- `processx::process$get_end_time()`
- `processx::process$get_error_connection()`
- `processx::process$get_error_file()`
- `processx::process$get_exe()`
- `processx::process$get_exit_status()`
- `processx::process$get_input_connection()`
- `processx::process$get_input_file()`
- `processx::process$get_memory_info()`
- `processx::process$get_name()`
- `processx::process$get_output_connection()`
- `processx::process$get_output_file()`
- `processx::process$get_pid()`
- `processx::process$get_poll_connection()`
- `processx::process$get_result()`
- `processx::process$get_start_time()`
- `processx::process$get_status()`
- `processx::process$get_username()`
- `processx::process$get_wd()`
- `processx::process$has_error_connection()`
- `processx::process$has_input_connection()`
- `processx::process$has_output_connection()`
- `processx::process$has_poll_connection()`
- `processx::process$interrupt()`
- `processx::process$is_alive()`
- `processx::process$is_incomplete_error()`
- `processx::process$is_incomplete_output()`
- `processx::process$is_supervised()`
- `processx::process$kill()`
- `processx::process$kill_tree()`
- `processx::process$poll_io()`
- `processx::process$print()`
- `processx::process$read_all_error()`
- `processx::process$read_all_error_lines()`
- `processx::process$read_all_output()`
- `processx::process$read_all_output_lines()`
- `processx::process$read_error()`
- `processx::process$read_error_bytes()`
- `processx::process$read_error_lines()`
- `processx::process$read_output()`
- `processx::process$read_output_bytes()`
- `processx::process$read_output_lines()`
- `processx::process$resume()`
- `processx::process$signal()`
- `processx::process$supervise()`
- `processx::process$suspend()`
- `processx::process$wait()`
- `processx::process$write_input()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-rscript_process-initialize}{}}} \subsection{\code{rscript_process$new()}}{ Create a new \code{Rscript} process. \subsection{Usage}{ \if{html}{\out{
}} \preformatted{rscript_process$new(options)} \if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{options}}{A list of options created via \code{\link[=rscript_process_options]{rscript_process_options()}}.} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-rscript_process-cleanup}{}}} \subsection{\code{rscript_process$cleanup()}}{ Delete the temporary files created for this \code{Rscript} process. Only call this method if you are sure that the process is done. If you don't call this method explicitly, the temporary files will be deleted when the process object is garbage collected. \subsection{Usage}{ \if{html}{\out{
}} \preformatted{rscript_process$cleanup()} \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-rscript_process-finalize}{}}} \subsection{\code{rscript_process$finalize()}}{ Clean up after an \code{Rscript} process, remove temporary files. \subsection{Usage}{ \if{html}{\out{
}} \preformatted{rscript_process$finalize()} \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-rscript_process-clone}{}}} \subsection{\code{rscript_process$clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}} \preformatted{rscript_process$clone(deep = FALSE)} \if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }